﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Security.Cryptography;
using System.Xml;
using System.Security.Cryptography.Xml;
using System.Xml.Serialization;
using System.IO;
using System.Security.Cryptography.X509Certificates;


namespace Acme.Licensing
{
  /// <summary>
  /// License verification class to be used in licensed applications.
  /// </summary>
  internal class LicenseManagerBase
  {
    public const string LicenseFileName = "Acme.License.xml";
    public const string PublicKeyFileName = "Acme.PublicKey.xml";
    public const string PublicKey = @"<RSAKeyValue><Modulus>yYTJwK3oc7HXeZsh6dCGgQJfHc+naP0XwmYZ7AEfZGGs88C7Nq34Q556iqqEtQCy+UIDYWqjLDZlGYOtvd9ePXFJ19ztTcK/VspOCgs0qmUtKPG7wDQZz44qOKpSBzVpQQePVBrJ9EfPCBgHJnh0YKD8BO80cXo9wuUxxE9BB0s=</Modulus><Exponent>AQAB</Exponent></RSAKeyValue>";
    public const string ProductName = "Acme Product #1";
    public const string TrialUse = "For trial and educational use only";
    public const string NoContact = "None";
    public const int TrialMaxCores = 2;
    public const string DateFormatString = "yyyy-MM-dd HH:mm:ss";
    public const string TrialExpiryDate = "2100-01-01 00:00:00";

    public static LicenseInfo GetTrialLicense()
    {
      LicenseInfo li = new LicenseInfo();
      li.ProductName = ProductName;
      li.LicenseId = Guid.Empty;
      li.LicenseeName = LicenseManagerBase.TrialUse;
      li.LicenseeContact = LicenseManagerBase.NoContact;
      li.ExpiryDate = LicenseManagerBase.TrialExpiryDate;
      li.MaxCores = LicenseManagerBase.TrialMaxCores;
      li.IsTrial = true;

      return li;
    }

    public static LicenseInfo GetLicenseInfo(string licenseFilePath, string publicKey)
    {
      // Get public key
      RSACryptoServiceProvider csp = new RSACryptoServiceProvider();
      csp.FromXmlString(publicKey);

      // Get license file
      if (!File.Exists(licenseFilePath))
      {
        throw new CryptographicException("License file not found.");
      }

      XmlDocument xmlDoc = new XmlDocument();
      try
      {
        xmlDoc.Load(licenseFilePath);
      }
      catch (XmlException ex)
      {
        throw new CryptographicException("Cannot load license file, bad XML. " + ex.Message);
      }

      // Find the "Signature" node and create a new XmlNodeList object. 
      XmlNodeList nodeList = xmlDoc.GetElementsByTagName("Signature");

      if (nodeList.Count <= 0)
      {
        throw new CryptographicException("No Signature element was found in the license document.");
      }

      if (nodeList.Count >= 2)
      {
        throw new CryptographicException("More that one signature was found for the license document.");
      }

      SignedXml signedXml = new SignedXml(xmlDoc);

      // Load the first <signature> node.   
      signedXml.LoadXml((XmlElement)nodeList[0]);

      // Check the signature and return the result. 
      if (!signedXml.CheckSignature(csp))
      {
        throw new CryptographicException("License document has invalid signature.");
      }

      // Derserialize LicenseInfo
      XmlSerializer xmlSerializer = new XmlSerializer(typeof(LicenseInfo));
      LicenseInfo li = (LicenseInfo)xmlSerializer.Deserialize(new XmlTextReader(new StringReader(xmlDoc.OuterXml)));

      // Check if the license has valid data
      CheckLicenseItem(li.ProductName, "product name");
      CheckLicenseItem(li.LicenseId, "license id");
      CheckLicenseItem(li.IssueDate, "issue date");
      CheckLicenseItem(li.ExpiryDate, "expiry date");
      CheckLicenseItem(li.LicenseeName, "licensee name");
      CheckLicenseItem(li.LicenseeContact, "licensee contact");
      CheckLicenseItem(li.MaxCores, "maximum number of processor cores");

      return li;
    }

    private static void CheckLicenseItem(object item, string caption)
    {
      if (item is string)
      {
        if (((string)item).Trim().Length == 0 || ((string)item).Trim().Length > 1024)
        {
          throw new ArgumentException(string.Format("License contains invalid {0}: {1}", caption, item));
        }
      }
      else if (item is int)
      {
        if (((int)item) <= 0)
        {
          throw new ArgumentException(string.Format("License contains invalid number of {0}: {1}", caption, item));
        }
      }
      else if (item is DateTime)
      {
        if (((DateTime)item) < new DateTime(2013, 1, 1) || ((DateTime)item) > new DateTime(2100, 1, 1))
        {
          throw new ArgumentException(string.Format("License contains invalid {0}: {1}", caption, item));
        }
      }
      else if (item is Guid)
      {
        if (((Guid)item) == Guid.Empty)
        {
          throw new ArgumentException(string.Format("License contains invalid {0}: {1}", caption, item));
        }
      }
      else
      {
        throw new ArgumentException("License contains invalid object: " + item);
      }
    }
  }
}
