﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Security.Cryptography;
using System.Xml;
using System.Security.Cryptography.Xml;
using System.Security.Cryptography.X509Certificates;
using System.IO;


namespace Acme.Licensing
{
  class Program
  {
    private const string CommandKey = "command";

    static void Hello()
    {
      Console.WriteLine("Acme license tool. Copyright 2013 Acme Inc.\r\n\r\n");
    }
    static void Usage()
    {
      Console.WriteLine("Usage: Acme.LicenseTool.exe [<command>] [parameters]\r\n" +
          "create [/license:<license file name>]\r\n" +
          "sign /cert:<certificate file name> /pass:<certificate password> [/license:<license file name>] \r\n" +
          "check [/license:<license file name>]\r\n" +
          "\r\n" +
          "Default command is 'check'. Default license file name is " + LicenseManager.LicenseFileName
          );
    }

    static Dictionary<string, string> ProcessParameters(string[] args)
    {
      Dictionary<string, string> argDict = new Dictionary<string, string>();
      for (int argIdx = 0; argIdx < args.Length; argIdx++)
      {
        if (!args[argIdx].StartsWith("/"))
        {
          if (argDict.ContainsKey(CommandKey))
          {
            Console.WriteLine(String.Format("Duplicate command. Only one of the following commands allowed: create, sign, check"));
            return null;
          }
          argDict.Add(CommandKey, args[argIdx]);
        }
        else
        {
          string[] argParts = args[argIdx].Split(':');
          string argValue = (argParts.Length >= 2 ? argParts[1] : string.Empty);
          if (argParts.Length > 2)
          {
            for (int argPartIdx = 2; argPartIdx < argParts.Length; argPartIdx++)
            {
              argValue += ":" + argParts[argPartIdx];
            }
          }

          // Strip quote marks
          if (argValue.StartsWith("\"") && argValue.EndsWith("\""))
          {
            argValue = argValue.Substring(1, argValue.Length - 2);
          }

          string argName = argParts[0];
          if (!argName.StartsWith("/"))
          {
            Console.WriteLine(String.Format("Invalid parameter: {0}", argName));
            return null;
          }

          argName = argName.Substring(1).ToLower();
          if (argDict.ContainsKey(argName))
          {
            Console.WriteLine(String.Format("Duplicate parameter: {0}", argName));
            return null;
          }

          argDict.Add(argName, argValue);
        }
      }
      return argDict;
    }

    static int CreateLicense(Dictionary<string, string> paramMap)
    {
      LicenseInfo li = LicenseManager.GetTrialLicense();
      li.IssueDate = DateTime.MinValue.ToString(LicenseManager.DateFormatString);

      string licenseFilePath = LicenseManager.LicenseFileName;
      if (paramMap.ContainsKey("license"))
      {
        licenseFilePath = paramMap["license"].Trim(new char[] { '"' });
      }

      LicenseManager.SaveLicense(li, licenseFilePath);

      Console.WriteLine("License saved: " + licenseFilePath);

      return 0;
    }

    static int SignLicense(Dictionary<string, string> paramMap)
    {
      string licenseFilePath = LicenseManager.LicenseFileName;
      if (paramMap.ContainsKey("license"))
      {
        licenseFilePath = paramMap["license"];
      }

      if (!paramMap.ContainsKey("cert"))
      {
        Console.WriteLine("Missing certificate parameter");
        return -1;
      }
      string certPath = paramMap["cert"];

      string certPassword = string.Empty;
      if (paramMap.ContainsKey("pass"))
      {
        certPassword = paramMap["pass"];
      }

      LicenseManager.SignLicense(licenseFilePath, certPath, certPassword);

      // Verify it right now to be 100% sure
      LicenseInfo li = LicenseManager.GetLicenseInfo(licenseFilePath, LicenseManager.PublicKey);
      CheckTrialLicenseItem("licensee name", li.LicenseeName, LicenseManager.TrialUse);
      CheckTrialLicenseItem("licensee contact", li.LicenseeContact, LicenseManager.NoContact);
      CheckTrialLicenseItem("maximum number of cores", li.MaxCores, LicenseManager.TrialMaxCores);
      CheckTrialLicenseItem("trial use only", li.IsTrial, true);

      Console.WriteLine("License signed: " + licenseFilePath);


      return 0;
    }

    static int CheckLicense(Dictionary<string, string> paramMap)
    {
      string licenseFilePath = LicenseManager.LicenseFileName;
      if (paramMap.ContainsKey("license"))
      {
        licenseFilePath = paramMap["license"];
      }

      if (!File.Exists(licenseFilePath))
      {
        Console.ForegroundColor = ConsoleColor.Red;
        Console.WriteLine("License file not found.");
        Console.ResetColor();
        Usage();
        return -1;
      }

      LicenseInfo li;
      try
      {
        li = LicenseManager.GetLicenseInfo(licenseFilePath, LicenseManager.PublicKey);
      }
      catch (CryptographicException ex)
      {
        Console.ForegroundColor = ConsoleColor.Red;
        Console.WriteLine(ex.Message);
        Console.ResetColor();
        return -1;
      }
      catch (ArgumentException ex)
      {
        Console.ForegroundColor = ConsoleColor.Red;
        Console.WriteLine(ex.Message);
        Console.ResetColor();
        return -1;
      }

      if (li.IsTrial)
      {
        Console.Write("This is a ");
        Console.ForegroundColor = ConsoleColor.Yellow;
        Console.Write("FREE license");
        Console.ResetColor();
        Console.WriteLine(" for trial and educational use only, see details below.\r\nPlease consider purchasing a commercial license.\r\n");
      }
      else
      {
        Console.Write("This is a valid ");
        Console.ForegroundColor = ConsoleColor.Green;
        Console.Write("commercial license");
        Console.ResetColor();
        Console.Write(", see details below\r\n");
      }

      PrintLicenseDetails(li);

      return 0;
    }

    static void PrintLicenseItem(string caption, object value, ConsoleColor c)
    {
      Console.Write(string.Format("{0}: ", caption));
      Console.ForegroundColor = c;
      Console.WriteLine(value);
      Console.ResetColor();
    }

    static void CheckTrialLicenseItem(string caption, object value, object trialValue)
    {
      if (value.Equals(trialValue))
      {
        Console.ForegroundColor = ConsoleColor.Yellow;
        Console.WriteLine("WARNING: field '{0}' has default value '{1}', is it a trial license?", caption, value);
        Console.ResetColor();
      }
    }

    static void PrintLicenseDetails(LicenseInfo li)
    {
      ConsoleColor c = (li.IsTrial ? ConsoleColor.Yellow : ConsoleColor.Green);

      PrintLicenseItem("Product", li.ProductName, c);
      PrintLicenseItem("Issued", li.IssueDate, c);
      PrintLicenseItem("Expires", li.ExpiryDate, c);
      PrintLicenseItem("Licensee", li.LicenseeName, c);
      PrintLicenseItem("Contact", li.LicenseeContact, c);
      PrintLicenseItem("Maximum number of cores", li.MaxCores, c);
    }

    static int Main(string[] args)
    {
      Hello();

      Dictionary<string, string> paramMap = ProcessParameters(args);

      if (!paramMap.ContainsKey(CommandKey) || paramMap[CommandKey].Equals("check"))
      {
        return CheckLicense(paramMap);
      }

      if (paramMap[CommandKey].Equals("create"))
      {
        return CreateLicense(paramMap);
      }

      if (paramMap[CommandKey].Equals("sign"))
      {
        return SignLicense(paramMap);
      }

      Console.WriteLine("Unknown command.\r\n\r\n");
      Usage();
      return -1;
    }
  }
}
